use rocket::{serde::{json::{Json, serde_json::json}, Deserialize, Serialize}, State};

use crate::{utils::ApiResponse, Action, ASender};
use crate::DBPool;

#[derive(Debug, Deserialize, Serialize, Clone)]
#[serde(crate = "rocket::serde")]
#[serde(rename_all = "camelCase")]
pub struct LightConfiguration {
    number: u32,
    direction_x: u32,
    direction_y: u32,
    direction_z: u32,
    tags: Option<String>
}

#[derive(Debug, Deserialize, Serialize, Clone)]
#[serde(crate = "rocket::serde")]
#[serde(rename_all = "camelCase")]
pub struct LightConfigurationRequest {
    lights: Vec<LightConfiguration>
}

#[post("/configure", data= "<data>")]
pub async fn configure(data: Json<LightConfigurationRequest>, db: &State<DBPool>, action: &State<ASender<Action>>) -> String {

    let conn = db.get();

    if conn.is_err() || conn.as_ref().ok().is_none() {
        return json!(ApiResponse {code: 500, message: "Clould not configure. Err: 1".to_string()}).to_string();
    }

    let conn = conn.ok().unwrap();

    let stmt = conn.prepare("SELECT configured from meta");

    if stmt.is_err() || stmt.as_ref().ok().is_none() {
        return json!(ApiResponse {code: 500, message: format!("Clould not configure. Err: 2, {:?}", stmt)}).to_string();
    }

    let mut stmt = stmt.ok().unwrap();

    struct Data {
        configured: bool,
    }

    let version_iter = stmt.query_map([], |row| {
        Ok(Data {
            configured: row.get(0)?
        })
    });

    if version_iter.is_err() || version_iter.as_ref().ok().is_none() {
        return json!(ApiResponse {code: 500, message: "Clould not configure. Err: 3".to_string()}).to_string();
    }

    let version_iter = version_iter.ok().unwrap();
    
    let mut recreate = false;

    for data in version_iter {
        if data.unwrap().configured  {
            println!("Already configured deleting current");
            recreate = true;
            break;
        }
    }

    drop(stmt);

    if recreate {
        let stmt = conn.execute("DELETE FROM configuration;", ());
        if stmt.is_err() || stmt.as_ref().ok().is_none() {
            return json!(ApiResponse {code: 500, message: format!("Clould not configure. Err: 4, {:?}", stmt)}).to_string();
        }
    }

    for light in data.lights.iter() {
        
        let mut tags: String = "".to_string();

        if light.tags.is_some() {
            tags = light.tags.as_ref().unwrap().to_string()
        }

        let stmt = conn.execute(
            "INSERT INTO configuration (ledcount, directionX, directionY, directionZ, tags) VALUES (?1, ?2, ?3, ?4, ?5);", 
            (light.number, light.direction_x, light.direction_y, light.direction_z, tags));

        if stmt.is_err() || stmt.as_ref().ok().is_none() {
            return json!(ApiResponse {code: 500, message: format!("Clould not configure. Err: 5, {:?}", stmt)}).to_string();
        }
    }

    let send = action.lock().unwrap().send(Action::Reload);

    if send.is_err() || send.ok().is_none() {
        return json!(ApiResponse {code: 500, message: "Failed to reload".to_string()}).to_string()
    }
    
    json!(ApiResponse {code: 200, message: "Configuration was successful".to_string()}).to_string()
}
